@@ -146,7 +146,7 @@ dlls = list()
 
 platform = sys.platform
 if platform == 'darwin':
-    dlls.append( ctypes.cdll.LoadLibrary('libssl.dylib') )
+    dlls.append( ctypes.cdll.LoadLibrary('libssl.32.dylib') )
 elif 'win' in platform:
     for d in ('libeay32.dll', 'libssl32.dll', 'ssleay32.dll'):
         try:
@@ -155,12 +155,9 @@ elif 'win' in platform:
             pass
 else:
     try:
-        dlls.append( ctypes.cdll.LoadLibrary('libssl.so.10') )
+        dlls.append( ctypes.cdll.LoadLibrary('libssl.so.1.1.0') )
     except OSError:
-        try:
-            dlls.append( ctypes.cdll.LoadLibrary('libssl.so.1.0.0') )
-        except OSError:
-            dlls.append( ctypes.cdll.LoadLibrary('libssl.so') )
+        dlls.append( ctypes.cdll.LoadLibrary('libssl.so') )
 
 class BIGNUM_Struct (ctypes.Structure):
     _fields_ = [ ("d",     ctypes.c_void_p),
@@ -196,13 +193,14 @@ def load_func( name, args, returns = ctypes.c_int):
 
 load_func( 'BN_new',   [],         BIGNUM )
 load_func( 'BN_free',  [ BIGNUM ], None )
-load_func( 'BN_init',  [ BIGNUM ], None )
 load_func( 'BN_clear', [ BIGNUM ], None )
 
 load_func( 'BN_CTX_new',  []        , BN_CTX )
-load_func( 'BN_CTX_init', [ BN_CTX ], None   )
 load_func( 'BN_CTX_free', [ BN_CTX ], None   )
 
+load_func( 'BN_set_flags', [ BIGNUM, ctypes.c_int ], None )
+BN_FLG_CONSTTIME = 0x04
+
 load_func( 'BN_cmp',      [ BIGNUM, BIGNUM ], ctypes.c_int )
 
 load_func( 'BN_num_bits', [ BIGNUM ], ctypes.c_int )
@@ -294,6 +292,7 @@ def calculate_x( hash_class, dest, salt, username, password ):
         username = six.b('')
     up = hash_class(username + six.b(':') + password).digest()
     H_bn_str( hash_class, dest, salt, up )
+    BN_set_flags(dest, BN_FLG_CONSTTIME)
 
 
 def update_hash( ctx, n ):
@@ -354,7 +353,7 @@ def get_ngk( hash_class, ng_type, n_hex, g_hex, ctx ):
 
 
 
-def create_salted_verification_key( username, password, hash_alg=SHA1, ng_type=NG_2048, n_hex=None, g_hex=None, salt_len=4 ):
+def create_salted_verification_key( username, password, hash_alg=SHA1, ng_type=NG_2048, n_hex=None, g_hex=None, salt_len=4, k_hex=None ):
     if ng_type == NG_CUSTOM and (n_hex is None or g_hex is None):
         raise ValueError("Both n_hex and g_hex are required when ng_type = NG_CUSTOM")
     s    = BN_new()
@@ -387,7 +386,7 @@ def create_salted_verification_key( username, password, hash_alg=SHA1, ng_type=N
 
 
 class Verifier (object):
-    def __init__(self,  username, bytes_s, bytes_v, bytes_A, hash_alg=SHA1, ng_type=NG_2048, n_hex=None, g_hex=None, bytes_b=None):
+    def __init__(self,  username, bytes_s, bytes_v, bytes_A, hash_alg=SHA1, ng_type=NG_2048, n_hex=None, g_hex=None, bytes_b=None, k_hex=None):
         if ng_type == NG_CUSTOM and (n_hex is None or g_hex is None):
             raise ValueError("Both n_hex and g_hex are required when ng_type = NG_CUSTOM")
         if bytes_b and len(bytes_b) != 32:
@@ -412,6 +411,8 @@ class Verifier (object):
 
         hash_class = _hash_map[ hash_alg ]
         N,g,k      = get_ngk( hash_class, ng_type, n_hex, g_hex, self.ctx )
+        if k_hex is not None:
+            BN_hex2bn(k, k_hex)
 
         self.hash_class = hash_class
         self.N          = N
@@ -432,6 +433,7 @@ class Verifier (object):
                 bytes_to_bn( self.b, bytes_b )
             else:
                 BN_rand(self.b, 256, 0, 0)
+            BN_set_flags(self.b, BN_FLG_CONSTTIME)
 
             # B = kv + g^b
             BN_mul(self.tmp1, k, self.v, self.ctx)
@@ -503,7 +505,7 @@ class Verifier (object):
 
 
 class User (object):
-    def __init__(self, username, password, hash_alg=SHA1, ng_type=NG_2048, n_hex=None, g_hex=None, bytes_a=None, bytes_A=None):
+    def __init__(self, username, password, hash_alg=SHA1, ng_type=NG_2048, n_hex=None, g_hex=None, bytes_a=None, bytes_A=None, k_hex=None):
         if ng_type == NG_CUSTOM and (n_hex is None or g_hex is None):
             raise ValueError("Both n_hex and g_hex are required when ng_type = NG_CUSTOM")
         if bytes_a and len(bytes_a) != 32:
@@ -529,6 +531,8 @@ class User (object):
 
         hash_class = _hash_map[ hash_alg ]
         N,g,k      = get_ngk( hash_class, ng_type, n_hex, g_hex, self.ctx )
+        if k_hex is not None:
+            BN_hex2bn(k, k_hex)
 
         self.hash_class = hash_class
         self.N          = N
@@ -543,6 +547,7 @@ class User (object):
         if bytes_A:
             bytes_to_bn( self.A, bytes_A )
         else:
+            BN_set_flags(self.a, BN_FLG_CONSTTIME)
             BN_mod_exp(self.A, g, self.a, N, self.ctx)
 
 
